@@ -1,9 +1,10 @@
 import datetime
-from typing import Union
+
+from python_utils import types
 
 
 def camel_to_underscore(name: str) -> str:
-    '''Convert camel case style naming to underscore style naming
+    '''Convert camel case style naming to underscore/snake case style naming
 
     If there are existing underscores they will be collapsed with the
     to-be-added underscores. Multiple consecutive capital letters will not be
@@ -31,7 +32,7 @@ def camel_to_underscore(name: str) -> str:
             elif i > 3 and not c.isupper():
                 # Will return the last 3 letters to check if we are changing
                 # case
-                previous = name[i - 3:i]
+                previous = name[i - 3 : i]
                 if previous.isalpha() and previous.isupper():
                     output.insert(len(output) - 1, '_')
 
@@ -40,8 +41,44 @@ def camel_to_underscore(name: str) -> str:
     return ''.join(output)
 
 
-def timesince(dt: Union[datetime.datetime, datetime.timedelta],
-              default: str = 'just now') -> str:
+def apply_recursive(
+    function: types.Callable[[str], str],
+    data: types.OptionalScope = None,
+    **kwargs
+) -> types.OptionalScope:
+    '''
+    Apply a function to all keys in a scope recursively
+
+    >>> apply_recursive(camel_to_underscore, {'SpamEggsAndBacon': 'spam'})
+    {'spam_eggs_and_bacon': 'spam'}
+    >>> apply_recursive(camel_to_underscore, {'SpamEggsAndBacon': {
+    ...     'SpamEggsAndBacon': 'spam',
+    ... }})
+    {'spam_eggs_and_bacon': {'spam_eggs_and_bacon': 'spam'}}
+
+    >>> a = {'a_b_c': 123, 'def': {'DeF': 456}}
+    >>> b = apply_recursive(camel_to_underscore, a)
+    >>> b
+    {'a_b_c': 123, 'def': {'de_f': 456}}
+
+    >>> apply_recursive(camel_to_underscore, None)
+    '''
+    if data is None:
+        return None
+
+    elif isinstance(data, dict):
+        return {
+            function(key): apply_recursive(function, value, **kwargs)
+            for key, value in data.items()
+        }
+    else:
+        return data
+
+
+def timesince(
+    dt: types.Union[datetime.datetime, datetime.timedelta],
+    default: str = 'just now',
+) -> str:
     '''
     Returns string representing 'time since' e.g.
     3 days ago, 5 hours ago etc.
